/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef BASECLASS_H
#define BASECLASS_H

#include <string>
#include <list>

#include "progelem.h"

using std::list;
using std::string;

typedef enum { NONE = 0, PUBLIC_TYPE, PROTECTED_TYPE, PRIVATE_TYPE, DUMMY_TYPE } DerivedType;

class ClassDecl;

class BaseClass : public ProgElem
{
 protected:
  bool is_virtual;
  DerivedType deriv_type;
  ClassDecl *class_decl; // the pointer to the ClassDecl for this BaseClass

 public:
  BaseClass(const string &name, bool virt, DerivedType t) :
    	ProgElem (name), is_virtual (virt), deriv_type (t), class_decl (0) {}

  const string &getClassName() const { return getText (); }
  bool isVirtual() const { return is_virtual; }
  DerivedType getDerivType() const { return deriv_type; }
  const string derivToString() const;
  ClassDecl *getClassDecl() const { return class_decl; }
  void setClassDecl(ClassDecl *cl) { class_decl = cl; }
  bool is_template_base_class() const { return getClassName().find ("<") != string::npos; }

  virtual void output(ostream &stream);
};

typedef list<BaseClass *> BaseClasses;
typedef BaseClasses::const_iterator BaseClassesIterator;

#endif // BASECLASS_H

