/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <assert.h>
#include <cstdlib>

#include "branchanalyser.h"
#include "multimethods.h"
#include "methods.h"
#include "multimethodtypes.h"
#include "mainerrormanager.h"
#include "classdecl.h"
#include "methparam.h"

static inline bool
check_conflicting_return_types(const MethDecl *meth, const Methods *methods);

int
BranchAnalyser::analyse_multimethod(const MultiMethod *mm)
{
  const Methods *methods = mm->getMethods ();

  if (! methods)
    {
      errormanager->print
          ("multi method with no branch", mm);
      return EXIT_FAILURE;
    }

  const ClassDecl *class_decl = mm->getClassDecl ();

  assert (class_decl); // TODO: this should be checked before?

  MethDecl *meth;

  for (MethodsIterator mit = methods->begin ();
       mit != methods->end (); ++mit)
    {
      meth = *mit;

      // TODO: this is temporary!!!
      meth->setClassDecl(class_decl);
      multi_method_types.addType(meth);

      if (! check_conflicting_return_types(meth, methods))
        return EXIT_FAILURE;
    }

  return EXIT_SUCCESS;
}

bool
check_conflicting_return_types(const MethDecl *meth, const Methods *methods)
{
  const MethDecl *m = 0;
  const DeclType *spec_param = meth->getSpecializingParam()->getDeclType();
  bool ret = true;

  for (MethodsIterator mit = methods->begin ();
       mit != methods->end (); ++mit)
    {
      m = *mit;

      if (m->equalsSignature(meth, 1) &&
          !m->getDeclType()->equals(meth->getDeclType()))
        {
          const DeclType *spec_param2 = m->getSpecializingParam()->getDeclType();

          if (! spec_param->unrelated(spec_param2))
            {
              errormanager->print
                  ("this branch return type (" + meth->getDeclType()->getTypeName() + ") conflicts ", meth);
              errormanager->print
                  ("with this branch return type (" + m->getDeclType()->getTypeName() + ")", m);
              ret = false;
            }
        }
    }

  return ret;
}
