/***************************************************************************
 *   Copyright (C) 2004 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "branchtype.h"

#include "decltype.h"
#include "methparams.h"
#include "classdecl.h"

BranchType::BranchType(const string &name,
                       const ClassDecl *receiver, const MethParam *target,
                       DeclType *ret, const MethParams *par, AccessType at,
                       const WithTokenInfo &wtk)
  : ToStringRep ((at != PUBLIC ? translate_access_type(at) + " " : "") +
         "( " + receiver->getClassName() +
         " x " + target->toStringNoName() +
         (par->size() > 1 ? ", " + par->toStringNoName(2) : "") +
         " ) -> " + ret->toString()),
    WithTokenInfo (wtk),
    MethodStruct (ret, par, at, name)
{
  setClassDecl(receiver);
}

BranchType::~BranchType()
{
}

const MethParam *
BranchType::getParam() const
{
  return getSpecializingParam();
}

const ClassDecl *
BranchType::getParamType() const
{
  return getParam()->getClassDecl();
}

bool
BranchType::equals(const BranchType &b) const
{
  return
    getDeclType()->equals(b.getDeclType()) &&
    accesstype == b.accesstype &&
    getReceiverType()->equals(b.getReceiverType()) &&
    getParam()->equals(b.getParam());
}

bool
BranchType::equalsReceiver(const BranchType &b) const
{
  return getReceiverType()->equals(b.getReceiverType());
}

bool
BranchType::isSubType(const BranchType &b, bool equal_ok, bool ok_private ) const
{
  return
    (getReceiverType()->equals(b.getReceiverType()) ||
      (ok_private || b.accesstype != PRIVATE)) &&
    getDeclType()->equals(b.getDeclType()) &&
    params->toStringNoName(2) == b.params->toStringNoName(2) &&
    getReceiverType()->isSubClass(b.getReceiverType(), equal_ok) &&
    getParam()->isSubClass(b.getParam(), equal_ok);
}
