/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef CLASSDECL_H
#define CLASSDECL_H

#include "progelem.h"
#include "baseclass.h"
#include "accesstype.h"

class MultiMethods;
class MultiMethod;
class MethDecl;
class ClassEnd;

// doublecpp: forward declarations, DO NOT MODIFY
class ProgElems; // file: progelems.h
class SourceModifier; // file: sourcemodifier.h
// doublecpp: end, DO NOT MODIFY

class ClassDecl : public ProgElem
{
 protected:
  BaseClasses *base_classes;
  MultiMethods *multimethods;
  ClassEnd *classend;
  bool is_struct; /// whether this class is actually a struct

 public:
  ClassDecl(const string &name, BaseClasses *bases) :
          ProgElem (name), base_classes (bases), multimethods (0),
          classend (0), is_struct(false)
        {}
  virtual ~ClassDecl() ;

  virtual void output(ostream &stream);

  /**
   * Returns a string with "class" or "struct" depending on the kind of class
   * @return a string with "class" or "struct" depending on the kind of class
   */
  const string getClassStruct() const { return (is_struct ? "struct" : "class"); }

  const string &getClassName() const { return getText(); }  
  const BaseClasses *getBaseClasses() const { return base_classes; }
  const MultiMethods *getMultiMethods() const { return multimethods; }
  bool contains_multimethods() const;
  const MultiMethod *retrieveMultiMethod(const string &) const;
  void addMultiMethod(MultiMethod *);
  bool equals(const ClassDecl *c) const
  { return getClassName() == c->getClassName(); }
  bool isSubClass(const ClassDecl *, bool equal_ok = true) const;
  bool isSubClass(const string &, bool equal_ok = true) const;
  bool isStruct() const { return is_struct; }
  void setStruct(bool s) { is_struct = s; }
  bool unrelated(const ClassDecl *) const;
  void addMethod(MethDecl *);
  bool hasMethod(const MethDecl *) const;
  const MethDecl *getMethod(const MethDecl *m) const;
  AccessType getMethodAccessType(const string &) const;
  void setClassEnd(ClassEnd *end) { classend = end; }
  ClassEnd *getClassEnd() const { return classend; }
// doublecpp: dispatch methods, DO NOT MODIFY
public:
virtual void dispatch_intern_add_item(ProgElems *);
virtual void dispatch_transform_elem(SourceModifier *, ProgElems * modified_decl, ProgElems * modified_meth);
// doublecpp: end, DO NOT MODIFY
};

struct ClassDecl_less
{
  typedef std::less<string> value_compare;

  value_compare value_comp;

  bool operator() (const ClassDecl *i1, const ClassDecl *i2) const
  { return value_comp (i1->getClassName (), i2->getClassName ()); }
};

#endif // CLASSDECL_H

