/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "classhierarchy.h"
#include "my_sstream.h"

#include <set>

using namespace std;

HierarchyNode::~HierarchyNode()
{
  for (ChildrenIterator cit = children->begin(); cit != children->end(); ++cit)
    delete (*cit);

  delete children;
}

bool
HierarchyNode::insert(const ClassDecl *c)
{
  if (c->isSubClass(base, false)) {
    bool inserted = false;
    for (ChildrenIterator cit = children->begin(); cit != children->end(); ++cit) {
      if ( (*cit)->insert (c) )
        inserted = true;
    }

    if (! inserted)
      children->push_back(new HierarchyNode (c));

    return true;
  } else if (base->isSubClass(c, false)) {
    // inverts the two nodes
    HierarchyNode *new_node = new HierarchyNode (*this);
    base = c;
    children = new Children;
    children->push_back (new_node);
    return true;
  } else
    return c->equals(base);
}

void
HierarchyNode::visit(OrderedHierarchy &new_hierarchy)
{
  Children queue;
  std::set<std::string> inserted;

  queue.push_back (this);
  HierarchyNode *extract;
  OrderedHierarchy hierarchy;

  while (! queue.empty ()) {
    extract = queue.front ();
    queue.pop_front ();
    hierarchy.push_front (extract->base);
    for (ChildrenIterator cit = extract->children->begin();
         cit != extract->children->end(); ++cit)
      queue.push_back (*cit);
  }

  // now remove possible duplicates, but ensure that the most
  // derived classes always appear first
  for (HierarchyIterator it = hierarchy.begin ();
       it != hierarchy.end (); ++it)
    {
      if (inserted.find ((*it)->getClassName()) ==
          inserted.end ())
        {
          inserted.insert ((*it)->getClassName());
          new_hierarchy.push_back (*it);
        }
    }
}

bool
ClassHierarchy::insert(const ClassDecl *c)
{
  return base->insert(c);
}

HierarchyIterator
ClassHierarchy::begin()
{
  if (! hierarchy.size())
    base->visit (hierarchy);

  return hierarchy.begin();
}

HierarchyIterator
ClassHierarchy::end()
{
  return hierarchy.end();
}

const string
ClassHierarchy::toString()
{
  ostringstream buf;

  for (HierarchyIterator it = begin(); it != end(); ++it)
    buf << "  " << (*it)->getClassName() << endl;

  return buf.str();
}
