/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

// a data structure to store in an "ordered" way the classes of
// a class hierarchy.
// visiting such a structure will result in visiting all classes
// of a hierarchy starting from the most derived up to the base class

#ifndef CLASSHIERARCHY_H
#define CLASSHIERARCHY_H

#include "classdecl.h"

#include <list>
#include <string>

using std::list;
using std::string;

typedef list<const ClassDecl *> OrderedHierarchy;
typedef OrderedHierarchy::const_iterator HierarchyIterator;

class HierarchyNode
{
protected:
  typedef list<HierarchyNode *> Children;
  typedef Children::iterator ChildrenIterator;

  const ClassDecl *base;
  Children *children;

public:
  HierarchyNode(const ClassDecl *c) : base (c), children (new Children) {}
  ~HierarchyNode();

  bool insert(const ClassDecl *);
  void visit(OrderedHierarchy &);
};

class ClassHierarchy
{
protected:
  HierarchyNode *base;
  OrderedHierarchy hierarchy;

public:
  ClassHierarchy(const ClassDecl *b) : base (new HierarchyNode (b)) {}
  ~ClassHierarchy() { delete base; }

  bool insert(const ClassDecl *);
  HierarchyIterator begin();
  HierarchyIterator end();

  const string toString();
};

#endif // CLASSHIERARCHY_H
