/***************************************************************************
 *   Copyright (C) 2004 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "dbtypegenerator.h"

#include "multimethodtypes.h"

static bool
check_LUB(const BranchType &candidate, const BranchType &top1, const BranchType &top2, const MultiMethodType &types);

DBTypeGenerator::DBTypeGenerator(const MultiMethodTypes &mtypes, MultiMethodTypes &ttypes, MultiMethodTypes &dtypes)
 : TopTypeGenerator(mtypes, ttypes), db_types (dtypes)
{
}

void
DBTypeGenerator::generate_type(const string &meth, const MultiMethodType &t)
{
  const MultiMethodType &meth_top_types = top_types.getType(meth);

  MultiMethodTypeIterator top1, top2, candidate, current;

  for (MultiMethodTypeIterator external = t.begin();
       external != t.end(); ++external)
    {
      candidate = external;

      for (MultiMethodTypeIterator internal = meth_top_types.begin(); internal != meth_top_types.end(); ++internal)
        {
          top1 = internal;
          if (!candidate->equals(*top1) && candidate->isSubType(*top1))
            {
              MultiMethodTypeIterator internal2 = internal;
              for (++internal2; internal2 != meth_top_types.end(); ++internal2)
                {
                  top2 = internal2;
                  if (!candidate->equals(*top2) && candidate->isSubType(*top2))
                    {
                      if (check_LUB(*candidate, *top1, *top2, t))
                        db_types.addType(*candidate);
                    }
                }
            }
        }
    }

  for (MultiMethodTypeIterator internal = meth_top_types.begin(); internal != meth_top_types.end(); ++internal)
    {
      db_types.addType(*internal);
    }
}

bool
check_LUB(const BranchType &candidate, const BranchType &top1, const BranchType &top2, const MultiMethodType &types)
{
  for (MultiMethodTypeIterator current = types.begin();
       current != types.end(); ++current)
    {
      if (!current->equals(candidate) && current->isSubType(top1) && current->isSubType(top2))
        {
          if (candidate.isSubType(*current))
            return false;
        }
    }

  return true;
}
