/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <assert.h>

#include "decltype.h"
#include "classdecl.h"

const string
DeclType::toString() const
{
  return (isConst () ? "const " : "") +
          getTypeName () +
          (isPointer () ? " *" : "");
}

const string
DeclType::toStringNoConst() const
{
  return getTypeName() + (isPointer () ? " *" : "");
}

// maintain the other feature of this declared type but
// change the name
const string
DeclType::toString(const string &new_name) const
{
  return (isConst () ? "const " : "") +
          new_name +
          (isPointer () ? " *" : "");
}

bool
DeclType::isBaseType (const DeclType *t)
{
  const string &tname = t->getTypeName ();

  return (tname == "int" ||
          tname == "bool" ||
          tname == "long" ||
          tname == "double" ||
          tname == "void" ||
          tname == "float" ||
          tname == "char");
}

bool
DeclType::isSubClass(const DeclType *t, bool equals_ok) const
{
  assert(class_decl);

  return (isConst() == t->isConst()) && class_decl->isSubClass(t->class_decl, equals_ok);
}

bool
DeclType::unrelated(const DeclType *t) const
{
  assert(class_decl);

  return (isConst() != t->isConst()) || class_decl->unrelated(t->class_decl);
}

