/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef DECLTYPE_H
#define DECLTYPE_H

#include "progelem.h"

using std::string;

class ClassDecl;

class DeclType : public ProgElem
{
 private:
  bool isconst;
  bool ispointer;
  const ClassDecl *class_decl; // the class representing this type

 public:
  DeclType(const string &s, bool isc, bool isp) :
        ProgElem (s), isconst (isc), ispointer (isp), class_decl(0) {}

  const string &getTypeName() const { return getText (); }
  bool isConst() const { return isconst; }
  bool isPointer() const { return ispointer; }

  const ClassDecl *getClassDecl() const { return class_decl; }
  void setClassDecl(const ClassDecl *c) { class_decl = c; }

  bool isSubClass(const DeclType *, bool equals_ok = true) const;
  bool unrelated(const DeclType *) const;

  bool equals(const DeclType *t) const
  {
    return ( t && getText () == t->getText () && (isconst == t->isconst) &&
             (ispointer == t->ispointer) );
  }

  virtual const string toString() const;
  const string toString(const string &) const;
  const string toStringNoConst() const;

  static bool isBaseType (const DeclType *t);
};

struct DeclType_less
{
  typedef std::less<string> value_compare;

  value_compare value_comp;

  bool operator() (const DeclType *i1, const DeclType *i2) const
  { return (value_comp (i1->toString(), i2->toString())); }
};


#endif // DECLTYPE_H

