/*
 * Copyright (C) 2002  Lorenzo Bettini <http://www.lorenzobettini.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>

#include "fileutil.h"

using std::ostream;
using std::ofstream;
using std::cerr;
using std::endl;

void
open_input_file(const string &file_name)
{
  FILE *in = 0;

  in = freopen (file_name.c_str (), "r", stdin);
  if (!in)
    {
      cerr << "Error in opening " << file_name
           << " for input" << endl ;
      exit (EXIT_FAILURE);
    }
}

bool
open_input_file_if_exists(const string &file_name)
{
  FILE *in = 0;

  in = freopen (file_name.c_str (), "r", stdin);
  if (!in)
    return false;

  return true;
}

bool
exist_file(const string &file_name)
{
  FILE *in = 0;

  in = fopen (file_name.c_str (), "r");
  if (!in)
    return false;

  fclose (in);

  return true;
}

ostream &
open_output_file(const string &file_name)
{
  ofstream *sout = new ofstream (file_name.c_str ());

  if ( ! *sout ) 
    {
      cerr << "Error in creating " << file_name << " for output" << endl ;
      exit (EXIT_FAILURE);
    }

  return *sout;
}

string
to_lower(const string &old)
{
  string lower = old;

  for (string::iterator s = lower.begin ();
       s != lower.end (); ++s)
    *s = tolower (*s);

  return lower;
}

string
get_file_path(const string &s)
{
  string::size_type pos_of_sep;

  pos_of_sep = s.rfind("/");
  if (pos_of_sep == string::npos)
    pos_of_sep = s.rfind("\\"); // try also with DOS path sep

  if (pos_of_sep == string::npos)
    return ""; // no path

  return s.substr (0, pos_of_sep + 1);
}

bool
contains_path(const string &s)
{
  return (get_file_path(s).size() > 0);
}

string
strip_file_path(const string &s)
{
  string::size_type pos_of_sep;

  pos_of_sep = s.rfind("/");
  if (pos_of_sep == string::npos)
    pos_of_sep = s.rfind("\\"); // try also with DOS path sep

  if (pos_of_sep == string::npos)
    return s; // no path

  return s.substr (pos_of_sep + 1);
}

string
strip_suffix(const string &s, const string &suff)
{
  if (!suff.size())
    return s;

  string::size_type pos_of_sep;

  pos_of_sep = s.rfind(suff);
  if (pos_of_sep == string::npos)
    return s; // no path

  return s.substr (0, pos_of_sep);
}

string
get_file_name_no_ext(const string &s)
{
  string::size_type pos_of_sep;

  pos_of_sep = s.rfind(".");

  if (pos_of_sep == string::npos)
    return s; // no path

  return s.substr (0, pos_of_sep);
}

