/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "includemodifier.h"
#include "mainglobalvars.h"
#include "sourcestructs.h"
#include "mainverbose.h"

void
IncludeModifier::modify_prog()
{
  const ProgElemsList &progelemslist = modified_decls->getProgElemsList();
  ProgElems *prog_elems;

  for (ProgElemsListIterator sit = progelemslist.begin ();
       sit != progelemslist.end (); ++sit)
    {
      prog_elems = *sit;
      const string old_name = prog_elems->getName() + "." + input_header_file_ext;
      const string &new_name = prog_elems->getFileName();
      if (old_name == new_name)
        continue;

      modify_source (prog_elems, old_name, new_name);
    }
    
  for (SourceStructsIterator ssit = orig_sources->begin();
       ssit != orig_sources->end(); ++ssit)
    {
      prog_elems = ssit->second;
      update_including_files (prog_elems);
    }
}

void
IncludeModifier::update_including_files(const ProgElems *source)
{
  const IncludeHeaders &include_headers = source->getIncludeHeaders();
  const IncludeHeader *inc_header;
  
  for (IncludeHeaderIterator hit = include_headers.begin();
       hit != include_headers.end(); ++hit)
    {
      inc_header = *hit;
      if (inc_header->isSystemInclude())
        continue;
      including_files[inc_header->getFileName()].push_back(source);
    }
}

void
IncludeModifier::modify_source(ProgElems *source, const string &old_name, const string &new_name)
{
  ProgElems *prog_elems;
  const ProgElemsList &progelemslist = modified_decls->getProgElemsList();

  for (ProgElemsListIterator sit = progelemslist.begin ();
       sit != progelemslist.end (); ++sit)
    {
      prog_elems = *sit;

      /*verbose->print
        ("header " + old_name + " changed in " + new_name + " in " +
         prog_elems->getFileName ());*/

      if (prog_elems->getFileName() == source->getFileName ()
          || ! prog_elems->contains_header (old_name) )
        continue;

      prog_elems->change_header (old_name, new_name);
      verbose->print
        ("header " + old_name + " changed in " + new_name + " in " +
         prog_elems->getFileName ());
    }

}
