/***************************************************************************
 *   Copyright (C) 2004 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "methodstruct.h"
#include "methparams.h"
#include "mainglobalvars.h"

MethodStruct::MethodStruct(DeclType *ret_type, const MethParams *pars, AccessType at, const string &name) :
  WithDeclType (ret_type), CanBeVirtual(assume_branch_virtual), method_name(name),
  params (pars), class_decl (0), accesstype (at)
{
}

MethodStruct::MethodStruct(const MethodStruct &m) :
  WithDeclType (m), method_name(m.method_name), params (m.params), class_decl (m.class_decl), accesstype (m.accesstype)
{
}

MethodStruct::~MethodStruct()
{
}

unsigned int
MethodStruct::getParamNumber() const
{
  return (params ? params->size() : 0);
}

MethParam *
MethodStruct::getSpecializingParam() const
{
  // we assume it is always the first one
  return *(params->begin ());
}

bool
MethodStruct::equalsSignature(const MethodStruct *meth, int index) const
{
  return (meth &&
      equalsSignature(meth->getMethodName(), meth->params, index));
}

bool
MethodStruct::equalsSignature(const string &name, const MethParams *parameters, int index) const
{
  return (getMethodName () == name &&
          params->equalsNoName (parameters, index));
}

bool
MethodStruct::equals(const MethodStruct *meth) const
{
  return (meth && getMethodName () == meth->getMethodName() &&
          params->equals (meth->params));
}
