/***************************************************************************
 *   Copyright (C) 2004 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef METHODSTRUCT_H
#define METHODSTRUCT_H

#include <string>

#include "withdecltype.h"
#include "canbevirtual.h"
#include "accesstype.h"

using std::string;

class ClassDecl;
class MethParams;
class MethParam;

/**
The base structure representing a method declaration

@author Lorenzo Bettini
*/
class MethodStruct : public WithDeclType, public CanBeVirtual
{
protected:
  string method_name;
  const MethParams *params;
  const ClassDecl *class_decl; // where this method is declared
  AccessType accesstype;

public:
  MethodStruct(DeclType *ret_type, const MethParams *pars, AccessType at, const string &name = "");
  MethodStruct(const MethodStruct &m);

  virtual ~MethodStruct();

  const string &getMethodName() const { return method_name; }
  const MethParams *getParams() const { return params; }
  unsigned int getParamNumber() const;
  void setMethodName(const string &n) { method_name = n; }
  const ClassDecl *getClassDecl() const { return class_decl; }
  void setClassDecl(const ClassDecl *c) { class_decl = c; }
  AccessType getAccessType() const { return accesstype; }
  bool isRestricted() const
  {
    return (accesstype ==  PROTECTED || accesstype == PRIVATE);
  }

  MethParam *getSpecializingParam() const;

  bool equals(const MethodStruct *meth) const;
  bool equalsSignature(const MethodStruct *meth, int index = 0) const;
  bool equalsSignature(const string &name, const MethParams *params, int index = 0) const;
};

#endif
