/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "methparams.h"

#include "my_sstream.h"

#define DEFAULT_PARAM_NAME "_param"

using std::ostringstream;

MethParams::~MethParams()
{
  for (MethParamsIterator it = begin (); it != end (); ++it)
    delete *it;
}

void
MethParams::addParam(MethParam *p)
{
  if (p->getText() == "")
    p->setText(counter.next(DEFAULT_PARAM_NAME));

  push_back(p);
}

const string
MethParams::toString() const
{
  ostringstream tr_params;

  MethParam *param;
  for (MethParamsIterator pit = begin ();
       pit != end (); )
   {
     param = *pit;
     tr_params << param->toString ();
     if (++pit != end ())
        tr_params << ", ";
   }

  return tr_params.str ();
}

// start from the pos-th parameter
const string
MethParams::toString(int pos) const
{
  ostringstream tr_params;

  MethParam *param;
  int counter = 1;
  for (MethParamsIterator pit = begin ();
       pit != end (); ++counter)
   {
     if (counter >= pos)
       {
         param = *pit;

         tr_params << param->toString ();
         if (++pit != end ())
           tr_params << ", ";
       }
     else
       ++pit;
   }

  return tr_params.str ();
}

// convert to string only the param types
const string
MethParams::toStringNoName(int pos) const
{
  ostringstream tr_params;

  MethParam *param;
  int counter = 1;
  for (MethParamsIterator pit = begin ();
       pit != end (); ++counter)
   {
     if (counter >= pos)
       {
         param = *pit;

         tr_params << param->toStringNoName ();
         if (++pit != end ())
           tr_params << ", ";
       }
     else
       ++pit;
   }

  return tr_params.str ();
}

// convert to string only the param names
const string
MethParams::toStringParamNames(int pos) const
{
  ostringstream tr_params;

  MethParam *param;
  int counter = 1;
  for (MethParamsIterator pit = begin ();
       pit != end (); ++counter)
   {
     if (counter >= pos)
       {
         param = *pit;

         tr_params << param->getText ();
         if (++pit != end ())
           tr_params << ", ";
       }
     else
       ++pit;
   }

  return tr_params.str ();
}
