/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef METHPARAMS_H
#define METHPARAMS_H

#include <list>

#include "methparam.h"
#include "uniquecounter.h"

using std::list;

class MethParams : protected list<MethParam *>
{
 UniqueCounter counter;

 public:
  typedef list<MethParam *>::const_iterator const_iterator;
  using list<MethParam *>::size;
  using list<MethParam *>::begin;
  using list<MethParam *>::end;

  virtual ~MethParams();

  void addParam(MethParam *p);

  bool equals(const MethParams *p) const
  {
    if (size () != p->size ())
      return false;

    for (const_iterator it = begin (), it2 = p->begin ();
         it != end (); ++it, ++it2)
      if (! (*it)->equals (*it2))
        return false;

    return true;
  }

  bool equalsNoName(const MethParams *p, int index = 0) const
  {
    if (size () != p->size ())
      return false;

    if (size () == 0)
      return true;

    const_iterator it = begin ();
    const_iterator it2 = p->begin ();

    for (int i = 0; i < index; ++i)
      {
        ++it;
        ++it2;
        if (it == end ())
          return true;
      }

    for (; it != end (); ++it, ++it2)
      if (! (*it)->equalsNoName (*it2))
        return false;

    return true;
  }

  virtual const string toString() const;
  const string toString(int) const;
  const string toStringNoName(int i = 1) const;
  const string toStringParamNames(int i = 1) const;
};

typedef MethParams::const_iterator MethParamsIterator;

#endif // METHPARAMS_H

