/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <assert.h>

#include "multimethod.h"
#include "methods.h"
#include "mainerrormanager.h"
#include "mainglobalvars.h"

MultiMethod::MultiMethod() :
  ProgElem (""), CanBeVirtual(assume_multimeth_virtual), methods (0), class_decl (0)
{
}

MultiMethod::MultiMethod(const string &s) :
  ProgElem (s), CanBeVirtual(assume_multimeth_virtual), methods (0), class_decl (0)
{
}

MultiMethod::~MultiMethod()
{
  if (methods) {
    delete methods;
  }
}

void
MultiMethod::add_method(MethDecl *m)
{
  if (! methods)
    methods = new Methods;

  if (! methods->addMethod (m))
   {
     errormanager->print
       ("duplicate method " + m->toString(), m);
     errormanager->print
       ("previous declaration was here",
        methods->getMethod(m));
     errormanager->exit_failure ();
   }
}

void
MultiMethod::updateMethodsName(const string &n)
{
  setText (n);

  if (! methods)
    return;

  for (MethodsIterator it = methods->begin (); it != methods->end (); ++it)
  {
    (*it)->setMethodName (n);
  }
}

bool
MultiMethod::hasMethod(const MethDecl *m) const
{
  return (methods && methods->find(m));
}

const MethDecl *
MultiMethod::getMethod(const MethDecl *m) const
{
  if (! methods)
    return 0;

  return methods->getMethod(m);
}

AccessType
MultiMethod::getAccessType() const
{
  assert (methods);

  return (*methods->begin())->getAccessType ();
}
