/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef MULTIMETHOD_H
#define MULTIMETHOD_H

#include "progelem.h"
#include "accesstype.h"
#include "canbevirtual.h"

#include <set>

using std::set;

class ClassDecl;
class MethDecl;
class Methods;

typedef set<string> InheritFromBases;
typedef InheritFromBases::const_iterator InheritFromBasesIterator;

// doublecpp: forward declarations, DO NOT MODIFY
class ProgElems; // file: progelems.h
class SourceModifier; // file: sourcemodifier.h
// doublecpp: end, DO NOT MODIFY

class MultiMethod : public ProgElem, public CanBeVirtual
{
 protected:
  Methods *methods; // the methods of this branch
  const ClassDecl *class_decl; // the class where this multimeth is declared
  InheritFromBases inheritfrombases;

 public:
  MultiMethod();
  MultiMethod(const string &s);
  virtual ~MultiMethod();

  const Methods *getMethods() const { return methods; }
  const string &getMethodName() const { return getText (); }
  AccessType getAccessType() const;

  void add_method(MethDecl *m);
  void updateMethodsName(const string &s);
  bool hasMethod(const MethDecl *) const;
  const MethDecl *getMethod(const MethDecl *m) const;

  void setClassDecl(const ClassDecl *c) { class_decl = c; }
  const ClassDecl *getClassDecl() const { return class_decl; }

  void addInheritFromBase(const string &s) { inheritfrombases.insert(s); }
  const InheritFromBases &getInheritFromBases() const { return inheritfrombases; }

// doublecpp: dispatch methods, DO NOT MODIFY
public:
virtual void dispatch_intern_add_item(ProgElems *);
virtual void dispatch_transform_elem(SourceModifier *, ProgElems * modified_decl, ProgElems * modified_meth);
// doublecpp: end, DO NOT MODIFY
};

#endif // MULTIMETHOD_H

