/***************************************************************************
 *   Copyright (C) 2004 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <assert.h>

#include "multimethodtypes.h"
#include "methdecl.h"
#include "classdecl.h"
#include "methparam.h"

using namespace std;

MultiMethodTypes::MultiMethodTypes()
{
}


MultiMethodTypes::~MultiMethodTypes()
{
}

void
MultiMethodTypes::addType(const BranchType &t)
{
  multi_method_table[t.getMethodName()].addBranchType(t);
  receiver_multi_method_table_table[t.getReceiverType()->getClassName()][t.getMethodName()].addBranchType(t);
  receiver_multi_method_table[t.getReceiverType()->getClassName()].addBranchType(t);
  target_multi_method_table[t.getParam()->getTypeName()].addBranchType(t);
}

void
MultiMethodTypes::addType(const MethDecl *meth)
{
  const ClassDecl *receiver_class = meth->getClassDecl();
  DeclType *ret_type = meth->getDeclType();
  const MethParam *param = meth->getSpecializingParam();

  assert(receiver_class);
  assert(ret_type);
  assert(param);

  const ClassDecl *param_class = param->getClassDecl();

  assert(param_class);

  addType(BranchType(meth->getMethodName(), receiver_class, param, ret_type,
                     meth->getParams(), meth->getAccessType(), *meth));
}

const string
MultiMethodTypes::toString() const
{
  string ret;

  for (MultiMethodTable::const_iterator it = multi_method_table.begin();
       it != multi_method_table.end(); ++it)
    {
      ret += it->first + " : " + it->second.toString() + "\n";
    }

  return ret;
}

// get all the types corresponding to the specified method name and receiver class
MultiMethodTypeIterator
MultiMethodTypes::begin(const string &m, const ClassDecl *c) const
{
  const string &class_name = c->getClassName();
  ClassMultiMethodTable::const_iterator it = receiver_multi_method_table_table.find(class_name);
  if (it != receiver_multi_method_table_table.end())
    {
      MultiMethodTypesIterator itt = it->second.find(m);
      if (itt != it->second.end())
        {
          return itt->second.begin();
        }
      else
        return dummy.end();
    }
  else
    return dummy.end();
}

MultiMethodTypeIterator
MultiMethodTypes::end(const string &m, const ClassDecl *c) const
{
  const string &class_name = c->getClassName();
  ClassMultiMethodTable::const_iterator it = receiver_multi_method_table_table.find(class_name);
  if (it != receiver_multi_method_table_table.end())
    {
      MultiMethodTypesIterator itt = it->second.find(m);
      if (itt != it->second.end())
        {
          return itt->second.end();
        }
      else
        return dummy.end();
    }
  else
    return dummy.end();
}

const string
MultiMethodTypes::toString(const string &m, const ClassDecl *c) const
{
  const string &class_name = c->getClassName();
  ClassMultiMethodTable::const_iterator it = receiver_multi_method_table_table.find(class_name);
  if (it != receiver_multi_method_table_table.end())
    {
      MultiMethodTypesIterator itt = it->second.find(m);
      if (itt != it->second.end())
        {
          return itt->second.toString();
        }
    }

  return "";
}

MultiMethodTypeIterator
MultiMethodTypes::begin(const ClassDecl *c) const
{
  MultiMethodTableIterator it = receiver_multi_method_table.find(c->getClassName());
  if (it != receiver_multi_method_table.end())
    {
      return it->second.begin();
    }

  return dummy.end();
}

MultiMethodTypeIterator
MultiMethodTypes::end(const ClassDecl *c) const
{
  MultiMethodTableIterator it = receiver_multi_method_table.find(c->getClassName());
  if (it != receiver_multi_method_table.end())
    {
      return it->second.end();
    }

  return dummy.end();
}

MultiMethodTypeIterator
MultiMethodTypes::begin_Param(const ClassDecl *p) const
{
  MultiMethodTableIterator it = target_multi_method_table.find(p->getClassName());
  if (it != target_multi_method_table.end())
    {
      return it->second.begin();
    }

  return dummy.end();
}

MultiMethodTypeIterator
MultiMethodTypes::end_Param(const ClassDecl *p) const
{
  MultiMethodTableIterator it = target_multi_method_table.find(p->getClassName());
  if (it != target_multi_method_table.end())
    {
      return it->second.end();
    }

  return dummy.end();
}

const string
MultiMethodTypes::toStringParam(const ClassDecl *c) const
{
  MultiMethodTableIterator it = target_multi_method_table.find(c->getClassName());
  if (it != target_multi_method_table.end())
    {
      return it->second.toString();
    }

  return "";
}

bool
MultiMethodTypes::contains_Param(const ClassDecl *cl) const
{
  return begin_Param(cl) != end_Param(cl);
}
