/***************************************************************************
 *   Copyright (C) 2004 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef MULTIMETHODTYPES_H
#define MULTIMETHODTYPES_H

#include <map>

using std::map;

#include "multimethodtype.h"

class MethDecl;

/**
@author Lorenzo Bettini
*/
class MultiMethodTypes
{
private:
  MultiMethodType dummy; // used for iterators

  typedef map<const string, MultiMethodType> MultiMethodTable;
  MultiMethodTable multi_method_table;

  typedef map<const string, MultiMethodTable> ClassMultiMethodTable;
  // key: receiver class, value: tables of all its multi method types
  ClassMultiMethodTable receiver_multi_method_table_table;

  // key: receiver class, value: all its multi method types
  MultiMethodTable receiver_multi_method_table;
  // key: target class, value: all the types of multi methods that use
  // this target class as the specializing param
  MultiMethodTable target_multi_method_table;

public:
  typedef MultiMethodTable::const_iterator MultiMethodTableIterator;

  MultiMethodTypes();

  ~MultiMethodTypes();

  void addType(const BranchType &t);
  void addType(const MethDecl *m);

  const string toString() const;

  const MultiMethodType &getType(const string &m) { return multi_method_table[m]; }

  int size() const { return multi_method_table.size(); }

  // get all the types corresponding to the specified receiver class
  MultiMethodTypeIterator begin(const ClassDecl *) const;
  MultiMethodTypeIterator end(const ClassDecl *) const;

  // get all the types corresponding to the specified param class
  MultiMethodTypeIterator begin_Param(const ClassDecl *) const;
  MultiMethodTypeIterator end_Param(const ClassDecl *) const;
  const string toStringParam(const ClassDecl *c) const;

  // whether there's a type with this class as a parameter
  bool contains_Param(const ClassDecl *cl) const;

  MultiMethodTableIterator begin() const { return multi_method_table.begin(); }
  MultiMethodTableIterator end() const { return multi_method_table.end(); }

  // get all the types corresponding to the specified method name and receiver class
  MultiMethodTypeIterator begin(const string &m, const ClassDecl *) const;
  MultiMethodTypeIterator end(const string &m, const ClassDecl *) const;
  const string toString(const string &m, const ClassDecl *) const;
};

typedef MultiMethodTypes::MultiMethodTableIterator MultiMethodTypesIterator;

#endif
