/***************************************************************************
 *   Copyright (C) 2004 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "parambinder.h"

#include <assert.h>
#include <cstdlib>

#include "classdecl.h"
#include "methparams.h"
#include "mainerrormanager.h"

int ParamBinder::analyse_method(const MethDecl *meth)
{
  MethParam *param = 0;
  const MethParams *params = meth->getParams();

  for (MethParamsIterator pit = params->begin(); 
      pit != params->end(); ++pit) {
      param = *pit;
      
      assert(param);
      
      const DeclType *decl_type = param->getDeclType();
      const string &param_type = decl_type->getTypeName ();
      
      // no class for base types so just go on
      if (DeclType::isBaseType(decl_type))
        continue;
      
      const ClassDecl *param_class = sources->retrieve_class_decl(param_type);
    
      if (! param_class)
      {
        errormanager->print
            ("can't find class for parameter " + param_type,
            decl_type);
        errormanager->exit_failure ();
      }
      else
        param->setClassDecl(param_class);
  }
  
  // now the return type
  DeclType *decl_type = meth->getDeclType();
  const string &ret_type = decl_type->getTypeName ();
  
  // no class for base types so just go on
  if (!DeclType::isBaseType(decl_type)) {
    const ClassDecl *ret_class = sources->retrieve_class_decl(ret_type);
  
    if (! ret_class)
    {
      errormanager->print
          ("can't find class for return type " + ret_type,
          decl_type);
      errormanager->exit_failure ();
    }
    else
      decl_type->setClassDecl(ret_class);
  }

  return EXIT_SUCCESS;
}
