/*
 * Copyright (C) 2003 Lorenzo Bettini <http://www.lorenzobettini.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef PROGELEM_H
#define PROGELEM_H

#include <string>
#include <iostream>
#include <functional>

#include "withtokeninfo.h"

using std::string;
using std::ostream;

// doublecpp: forward declarations, DO NOT MODIFY
class ProgElems; // file: progelems.h
class SourceModifier; // file: sourcemodifier.h
// doublecpp: end, DO NOT MODIFY

class ProgElem : public WithTokenInfo
{
 protected:
  string text;

 public:
  ProgElem() {}
  ProgElem(const string &s);
  ProgElem(const ProgElem &p);

  virtual ~ProgElem() {}

  const string &getText () const { return text; }
  void setText(const string &t) { text = t; }

  virtual void output(ostream &stream);
  virtual bool isToBePreProcessed () const;
  virtual const string toString() const;
// doublecpp: dispatch methods, DO NOT MODIFY
public:
virtual void dispatch_intern_add_item(ProgElems *);
virtual void dispatch_transform_elem(SourceModifier *, ProgElems * modified_decl, ProgElems * modified_meth);
// doublecpp: end, DO NOT MODIFY
};

/**
* less operator (based on getText method) for ProgElem pointers.
*/
struct ProgElem_less
{
  typedef std::less<string> value_compare;

  value_compare value_comp;

  bool operator() (const ProgElem *i1, const ProgElem *i2) const
  { return value_comp (i1->getText (), i2->getText ()); }
};

ostream &
operator<<(ostream &stream, const ProgElem &item) ;

#endif // PROGELEM_H
