/*
 * Copyright (C) 2003 Lorenzo Bettini <http://www.lorenzobettini.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <assert.h>
#include <stdlib.h>

#include "progelems.hpp"
#include "withmethdecls.h"
#include "mainerrormanager.h"
#include "classend.h"
#include "multimethods.h"
#include "classdecl.h"

ProgElems::~ProgElems()
{
  iterator it;

  for (it = begin (); it != end (); ++it)
    delete *it;
}

void
ProgElems::add_item(ProgElem *item)
{
  if (! item)
    return;

  push_back (item);

  intern_add_item_DB (item);
}

void
ProgElems::add_item_front(ProgElem *item)
{
  if (! item)
    return;

  push_front (item);

  intern_add_item_DB (item);
}

void
ProgElems::add_class_decl(ClassDecl *cl)
{
  if (class_elems.size())
  {
    ClassDecl *current_class = class_elems.back ();
    if (! current_class->getClassEnd())
      {
        errormanager->print
          ("incorrect new class definition", cl);
        errormanager->print
          ("previous class was not closed", current_class);
        errormanager->exit_failure();
      }
  }

  class_elems.push_back (cl);
}

void
ProgElems::add_multimethod(MultiMethod *multimethod)
{
  if (class_elems.size () == 0)
    {
      errormanager->print
        ("multimethods have to be defined inside a class", multimethod);
      errormanager->exit_failure();
    }

  ClassDecl *current_class = class_elems.back ();

  current_class->addMultiMethod (multimethod);
}

void
ProgElems::add_classend(ClassEnd *class_end)
{
      if (class_elems.size () == 0)
        {
          errormanager->print
           ("not a correct class ending", class_end);
          exit (EXIT_FAILURE);
        }
      class_end->setClassDecl (class_elems.back ());
}

void
ProgElems::add_include(IncludeHeader *h)
{
  includeheaders.add (h);
}

void
ProgElems::generate()
{
  iterator it;

  for (it = begin (); it != end (); ++it)
    (*it)->output (stream);
}

ClassDecl *
ProgElems::retrieve_class_decl(const string &c) const
{
  return class_elems.retrieve_class_decl (c);
}

// whether this source contains classes with multimethods
bool
ProgElems::contains_multimethods() const
{
  const ClassDecls &classes = getClasses ();

  //assert (classes.size () > 0);

  for (ClassDeclIterator cit = classes.begin ();
       cit != classes.end (); ++cit)
    {
      if ((*cit)->contains_multimethods())
        return true;
    }

  return false;
}

void
ProgElems::change_header(const string &old, const string &new_name)
{
  includeheaders.change_header (old, new_name);
}
