/*
 * Copyright (C) 2002 Lorenzo Bettini <http://www.lorenzobettini.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef PROGELEMS_H
#define PROGELEMS_H

#include <iostream>
#include <functional>
#include <string>
#include <list>

#include "classdecls.h"
#include "withmethdecls.h"
#include "forwarddecls.h"
#include "includeheaders.h"

using std::string;
using std::list;

class ClassEnd;

// contains all the method declarations (previously) generated by the
// this preprocessor, doublecpp
typedef list<WithMethDecls *> GeneratedMethDeclList;
typedef GeneratedMethDeclList::const_iterator GeneratedMethDeclListIt;

// doublecpp: forward declarations, DO NOT MODIFY
class ClassDecl; // file: classdecl.h
class ClassEnd; // file: classend.h
class ForwardDecls; // file: forwarddecls.h
class IncludeHeader; // file: includeheader.h
class MultiMethod; // file: multimethod.h
class ProgElem; // file: progelem.h
class WithMethDecls; // file: withmethdecls.h
// doublecpp: end, DO NOT MODIFY

#line 43 "./progelems.h"
class ProgElems : public list<ProgElem *>
{
 protected:
  ClassDecls class_elems;
  GeneratedMethDeclList generated_meth_list;
  ForwardDecls forwarddecls;
  IncludeHeaders includeheaders;

  string name;
  string file_name;
  ostream &stream;

  bool to_generate;

 public:
  ProgElems(const string &n, const string &file, ostream &s) :
    name (n), file_name (file), stream (s), to_generate (false) {}
  virtual ~ProgElems() ;

 protected:
// doublecpp: method branches, DO NOT MODIFY
#line 64 "./progelems.h"
virtual void intern_add_item(ProgElem * p);
#line 68 "./progelems.h"
virtual void intern_add_item(ClassDecl * c);
#line 72 "./progelems.h"
virtual void intern_add_item(MultiMethod * b);
#line 76 "./progelems.h"
virtual void intern_add_item(ClassEnd * c);
#line 80 "./progelems.h"
virtual void intern_add_item(WithMethDecls * w);
#line 84 "./progelems.h"
virtual void intern_add_item(ForwardDecls * forw);
#line 88 "./progelems.h"
virtual void intern_add_item(IncludeHeader * i);
public:
void _forward_intern_add_item(ClassDecl * c)
{
  intern_add_item(c);
};

void _forward_intern_add_item(ClassEnd * c)
{
  intern_add_item(c);
};

void _forward_intern_add_item(ForwardDecls * forw)
{
  intern_add_item(forw);
};

void _forward_intern_add_item(IncludeHeader * i)
{
  intern_add_item(i);
};

void _forward_intern_add_item(MultiMethod * b)
{
  intern_add_item(b);
};

void _forward_intern_add_item(ProgElem * p)
{
  intern_add_item(p);
};

void _forward_intern_add_item(WithMethDecls * w)
{
  intern_add_item(w);
};

protected:
virtual void intern_add_item_DB(ProgElem * p);
#line 91 "./progelems.h"
// doublecpp: end, DO NOT MODIFY


 public:
  virtual void add_item(ProgElem *);
  virtual void add_item_front(ProgElem *);
  virtual void add_class_decl(ClassDecl *);
  virtual void add_multimethod(MultiMethod *);
  virtual void add_classend(ClassEnd *);
  virtual void add_include(IncludeHeader *);
  virtual void generate();

  const ClassDecls &getClasses() const { return class_elems; }
  const GeneratedMethDeclList *getGeneratedMethDeclList() const
  { return &generated_meth_list; }
  const ForwardDecls &getForwardDecls() const
  { return forwarddecls; }
  ClassDecl *retrieve_class_decl(const string &) const;
  const IncludeHeaders &getIncludeHeaders() const { return includeheaders; }
  bool contains_header(const string &h) const { return includeheaders.contains_header (h); }
  void change_header(const string &old, const string &new_name);

  bool contains_multimethods() const;

  bool isToGenerate() const { return to_generate; }
  void setToGenerate(bool b) { to_generate = b; }

  const string &getName() const { return name; }
  const string &getFileName() const { return file_name; }
};

typedef ProgElems::const_iterator ProgElemsIterator;

#endif // PROGELEMS_H
