/***************************************************************************
 *   Copyright (C) 2003-2006 by Lorenzo Bettini                            *
 *   http://www.lorenzobettini.it                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include <cstdlib>
#include "programanalyser.h"

#include "multimethods.h"
#include "methods.h"
#include "multimethodtypes.h"
#include "classdecl.h"

int ProgramAnalyser::analyse()
{
  const ProgElemsList &progelemslist = sources->getProgElemsList ();

  for (ProgElemsListIterator sit = progelemslist.begin ();
       sit != progelemslist.end (); ++sit)
    if (analyse_source (*sit) == EXIT_FAILURE)
      return EXIT_FAILURE;

  return EXIT_SUCCESS;
}

int ProgramAnalyser::analyse_source(ProgElems *source)
{
  const ClassDecls &classes = source->getClasses ();

  if (classes.size () == 0)
    return EXIT_SUCCESS; // TODO: should be an error?

  const MultiMethods *multimethods = 0;

  for (ClassDeclIterator cit = classes.begin ();
       cit != classes.end (); ++cit)
  {
    multimethods = (*cit)->getMultiMethods ();
    if (multimethods)
      if (analyse_multimethods (multimethods) == EXIT_FAILURE)
        return EXIT_FAILURE;
  }

  return EXIT_SUCCESS;
}

int ProgramAnalyser::analyse_multimethods(const MultiMethods *multimethods)
{
  MultiMethod *mm;

  for (MultiMethodsIterator bit = multimethods->begin ();
       bit != multimethods->end (); ++bit)
  {
    mm = *bit;
    if (analyse_multimethod (mm) == EXIT_FAILURE)
      return EXIT_FAILURE;
  }

  return EXIT_SUCCESS;
}

int ProgramAnalyser::analyse_multimethod(const MultiMethod *mm)
{
  const Methods *methods = mm->getMethods ();

  if (! methods)
    return EXIT_SUCCESS; // TODO: this should be an error!

  return analyse_methods (methods);
}

int ProgramAnalyser::analyse_methods(const Methods *methods)
{
  MethDecl *meth;

  for (MethodsIterator mit = methods->begin (); mit != methods->end (); ++mit)
  {
    meth = *mit;
    if (analyse_method (meth) == EXIT_FAILURE)
      return EXIT_FAILURE;
  }

  return EXIT_SUCCESS;
}

int ProgramAnalyser::analyse_method(const MethDecl *meth)
{
  return EXIT_SUCCESS;
}


