/***************************************************************************
 *   Copyright (C) 2003-2006 by Lorenzo Bettini                            *
 *   http://www.lorenzobettini.it                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef PROGRAMANALYSER_H
#define PROGRAMANALYSER_H

#include "sourcestructs.h"

class MultiMethods;

/**
Generic class for inspecting the elements of a program (provides the
analyse_xxx methods that inspects all the subelements)

	@author Lorenzo Bettini <http://www.lorenzobettini.it>
*/
class ProgramAnalyser
{
  protected:
    SourceStructs *sources;

public:
    ProgramAnalyser(SourceStructs *src) :
      sources (src) {}

    virtual ~ProgramAnalyser() {}

    /**
     * Calls analyse_source on each ProgElems in the SourceStructs
     * @return EXIT_SUCCESS or EXIT_FAILURE
     */
    virtual int analyse() ;

    /**
     * Calls analyse_multimethods on all multimethods of each
     * class in ProgElems
     * @param source The ProgElems to analyse
     * @return EXIT_SUCCESS or EXIT_FAILURE
     */
    virtual int analyse_source(ProgElems *source);

    /**
     * Calls analyse_multimethod on each multimethod contained
     * in multimethods
     * @param multimethods The set of multimethods to analyse
     * @return EXIT_SUCCESS or EXIT_FAILURE
     */
    virtual int analyse_multimethods(const MultiMethods *multimethods);

    /**
     * Calls analyse_methods on each element of multimethod
     * @param multimethod The multimethod to analyse
     * @return EXIT_SUCCESS or EXIT_FAILURE
     */
    virtual int analyse_multimethod(const MultiMethod *multimethod);

    /**
     * Calls analyse_method on each element of methods
     * @param  methods The branches of the multimethod to analyse
     * @return EXIT_SUCCESS or EXIT_FAILURE
     */
    virtual int analyse_methods(const Methods *methods);

    /**
     * Analyses the single branch of a multimethod.
     *
     * The default implementation does nothing: the subclasses should
     * override this method
     *
     * @param  methdecl the branch to analyse
     * @return EXIT_SUCCESS or EXIT_FAILURE
     */
    virtual int analyse_method(const MethDecl *methdecl);
};

#endif
