/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef SOURCEANALYSER_H
#define SOURCEANALYSER_H

#include <set>

#include "programanalyser.h"
#include "progelems.hpp"

using std::set;

class MultiMethods;

/**
 * Visit all the sources (recursively) involved in the analysis of a specific input
 * file:
 * - the header files included
 * - the header files inferred by class names (provided --dont-infer-headers
 * is not given at command line)
 * - previously generated outputs (if any)
 *
 *       @author Lorenzo Bettini <http://www.lorenzobettini.it>
 */
class SourceAnalyser : public ProgramAnalyser
{
 protected:
  typedef set<string> analysed_items_container;

  analysed_items_container analysed_files;
  analysed_items_container analysed_classes;
  unsigned int scanned_files;
  unsigned int scanned_classes;

  virtual int analyse(const string &file_name, bool previous_output=false);
  virtual int analyse_class(const ClassDecl *, ProgElems *);
  virtual int analyse_header(const IncludeHeader *);
  virtual int analyse_header_file(const string &);
  virtual int analyse_base_classes(const ClassDecl *);
  virtual int analyse_type_name(const string &, const ProgElem *);
  virtual int analyse_method(const MethDecl *);
  virtual int analyse_previous_outputs();
  virtual int analyse_previous_output(const ProgElems *);

 public:
  SourceAnalyser(SourceStructs *s) :
    ProgramAnalyser (s), scanned_files (0), scanned_classes (0) {}
  virtual ~SourceAnalyser() {}

  virtual int analyse_program(const string &file_name);
  virtual void generate();

  unsigned int getAnalysedFileNum() const { return scanned_files; }
  unsigned int getAnalysedClassNum() const { return scanned_classes; }
};

#endif // SOURCEANALYSER_H

