/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <fstream>
#include "my_sstream.h"

#include "sourcegenerator.h"
#include "sourcestructs.h"
#include "mainprogelems.h"
#include "mainglobalvars.h"
#include "progelems.hpp"
#include "fileutil.h"
#include "mainverbose.h"

using std::ifstream;
using std::ostringstream;

void
SourceGenerator::generate()
{
  generate_sources (modified_declarations);
  generate_sources (modified_sources);
}

void
SourceGenerator::generate_sources(const SourceStructs *sources)
{
  const ProgElemsList &progelemslist = sources->getProgElemsList ();

  for (ProgElemsListIterator sit = progelemslist.begin ();
       sit != progelemslist.end (); ++sit)
    generate_sources (*sit);
}

void
SourceGenerator::generate_sources(const ProgElems *sources)
{
  if (sources->size () == 0 || ! sources->isToGenerate())
    return; // it is a fake source, due to invade_target option

  const string file_name =
    main_path +
    sources->getFileName ();
  verbose->print_noln ("generating " + file_name + "... ");

  ostringstream to_generate;

  for (ProgElemsIterator pit = sources->begin ();
       pit != sources->end (); ++pit)
    {
      to_generate << (*pit)->toString ();
    }

  const string &to_generate_string = to_generate.str ();

  bool is_to_generate = true;
  ifstream input (file_name.c_str ());

  if (input)
    {
      ostringstream original_output;
      string line;

      while (true)
      {
        getline (input, line);
        original_output << line;
        if (input.eof ())
          break;
        else
          original_output << endl;
      }

      input.close ();

      const string &original_output_string = original_output.str();

      if (original_output_string == to_generate_string)
        is_to_generate = false;
    }

    if (! is_to_generate && ! force_regeneration)
      {
        verbose->print ("no changes");
      }
    else
      {
        ostream &output = open_output_file (file_name);

        output << to_generate_string;
        output.flush ();
        verbose->print ("done");
      }
}
