/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef SOURCEMODIFIER_H
#define SOURCEMODIFIER_H

#include <string>
#include <set>

using std::string;
using std::set;
using std::less;

class SourceStructs;
class MultiMethod;
class ClassEnd;
class MethDecl;
class Methods;
class WithTokenInfo;
class ProgElem;
class ProgElems;
class MethParam;
class MethParams;

#define COMMENT_TAG "// doublecpp: "
#define FORWARD_DECL "forward declarations"
#define END_TAG "end"
#define DISPATCH_METHS "dispatch methods"
#define METHOD_BRANCHES "method branches"

#include "main_method.h"
#include "classdecl.h"

typedef set<const ClassDecl *, ProgElem_less > GeneratedForwardDecls;

class GeneratedIncludes;
class MultiMethodTypes;
class MultiMethodType;
class BranchType;

class SourceModifier
{
 protected:
  typedef set<string, std::less<string> > BaseOfSpecArgs;

  SourceStructs *sourcestructs;
  SourceStructs *modified_declarations;
  SourceStructs *modified_methods;
  GeneratedIncludes *generated_includes;
  const MultiMethodTypes &multi_method_types;
  const MultiMethodTypes &db_types;
  const MultiMethodTypes &extended_types;

  bool visit_only_targets;

  branch transform_elem
    void (ProgElem *elem, ProgElems *modified_decl, ProgElems *modified_meth);
    void (MultiMethod *br, ProgElems *modified_decl, ProgElems *modified_meth);
    void (ClassDecl *cl, ProgElems *modified_decl, ProgElems *modified_meth);
    void (ClassEnd *cend, ProgElems *modified_decl, ProgElems *modified_meth);
    void (IncludeHeader *ih, ProgElems *modified_decl, ProgElems *modified_meth);
  endbranch

 protected:
  virtual void transform_multimethod(MultiMethod *,
                               ProgElems *, ProgElems *);
  virtual void transform_class_decl(ClassDecl *, ProgElems *);
  virtual void transform_classend(ClassEnd *, ProgElems *,
                                  ProgElems *) = 0;
  virtual void handle_restricted_methods(ProgElems *, const string &meth, const ClassDecl *cl) {}
  virtual void transform_includeheader(IncludeHeader *, ProgElems *);

  void add_main_method(const MultiMethod *multimethod,
                               const BranchType &btype,
                               ProgElems *modified_decl,
                               ProgElems *modified_meth);
  virtual void generate_main_method(ProgElems *modified_meth,
                                    const BranchType &bt) = 0;
  void fill_main_method_generator(main_method_gen_class &m_gen, const BranchType &bt);

  virtual const string change_method_name(const MethDecl *);
  virtual const string change_method_name(const string &);
  virtual const string change_main_method_name(const string &);
  virtual const string dispatch_method_name(const MethDecl *);
  virtual const string dispatch_method_name(const string &method_name, const MethParam *p);
  void generate_using(const ClassDecl *, MultiMethod *) const;
  virtual void generate_usings(ProgElems *modified_decl, const MultiMethod *);
  virtual void generate_include_header(const string &cl, ProgElems *source,
                                       bool use_output_ext = true);
  virtual const string generate_line_info(const string &,
                                          const WithTokenInfo *,
                                          bool last = false);
  virtual void insert_line_info(ProgElems *, const string &, const WithTokenInfo *,
                                bool last = false);
  virtual const string generate_original_file_info(const string &);
  virtual bool should_invade_target(const string &);
  virtual bool should_use_output_extension(const string &);
  bool should_modify(ProgElems *);
  ProgElems *createModifiedDeclSource(ProgElems *, bool);
  ProgElems *createModifiedMethSource(ProgElems *, bool);

  static const string generate_comment_tag(const string &s);
  static void fill_forward_decl_param(const BranchType &btype, GeneratedForwardDecls &forward_decls, int pos=1);
  virtual void fill_target_forward_decls(const ClassDecl *, GeneratedForwardDecls &) = 0;
  void generate_include(const string &cl, ProgElems *source,
                        bool use_output_ext = true);

 public:
  SourceModifier(SourceStructs *s, SourceStructs *src, SourceStructs *m,
                 const MultiMethodTypes &mtypes,
                 const MultiMethodTypes &db_t,
                 const MultiMethodTypes &ext);
  virtual ~SourceModifier();

  int virtual analyse();
  virtual void analyse_source(ProgElems *);
};

#endif // SOURCEMODIFIER_H

