/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <assert.h>

#include "sourcemodifiermodular.h"

#include "methods.h"
#include "classend.h"
#include "sourcestructs.h"
#include "rtti_check.h"
#include "my_sstream.h"
#include "main_method.h"
#include "classhierarchy.h"
#include "multimethodtypes.h"
#include "mainverbose.h"
#include "normaltext.h"
#include "methparams.h"

using std::ostringstream;
using std::endl;

void
SourceModifierModular::generate_main_method(ProgElems *modified_meth,
                                    const BranchType &btype)
{
  const MethParam *spec_param = btype.getParam();
  const string &top_spec_arg = spec_param->getDeclType()->getTypeName();
  const MethParams *params = btype.getParams();
  const ClassDecl *spec_param_class = btype.getParamType();
  const string &ret_type = btype.getRetType()->getTypeName();
  const string &method_name = btype.getMethodName();

  ClassHierarchy class_hierarchy (spec_param_class);
  ostringstream ret_string;

  gather_methods (class_hierarchy, btype);

  HierarchyIterator it, saved;
  rtti_check_gen_class rtti_gen;

  bool method_is_function = (ret_type != "void");
  string main_param = spec_param->getText ();
  const string &other_params =
    (params->size () > 1 ? ", " + params->toStringParamNames (2) : "");
  const string changed_method_name = change_method_name(method_name);

  rtti_gen.set_ret_return ((method_is_function ? "return " : ""));
  rtti_gen.set_exit_return ((method_is_function ? "" : "\n    return;"));
  rtti_gen.set_other_params (other_params);
  rtti_gen.set_param (main_param);
  rtti_gen.set_meth_name (changed_method_name);

  verbose->print("Class hierarchy for parameters of " +
    change_main_method_name (method_name) +
    "(" + params->toString() + ")");
  verbose->print(class_hierarchy.toString());

  for (it = class_hierarchy.begin (); it != class_hierarchy.end(); )
    {
       saved = it;

       generate_include_header
           ((*saved)->getText (), modified_meth,
            should_use_output_extension(top_spec_arg));

       if ( ++it == class_hierarchy.end() ) // last element
        {
          // it will be the same of the parameter of the main method
          ret_string << (method_is_function ? "return " : "");
          ret_string << change_method_name (method_name) << "("
                     << main_param << other_params << ");\n";
        }
       else
        {
         rtti_gen.set_real_class
           ((spec_param->isConst() ? "const " : "") + (*saved)->getText () + " *");
         rtti_gen.generate_rtti_check (ret_string, 2); // indent 2
        }
    }

  main_method_gen_class main_meth_gen;
  ostringstream main_meth_string;

  fill_main_method_generator (main_meth_gen, btype);

  main_meth_gen.set_body (ret_string.str ());

  main_meth_gen.generate_main_method (main_meth_string);

  modified_meth->add_item
    (new NormalText (main_meth_string.str ()));
}

void
SourceModifierModular::gather_methods(ClassHierarchy &hier, const BranchType &btype)
{
  const ClassDecl *cl = btype.getReceiverType();
  const string &method_name = btype.getMethodName();

  for (MultiMethodTypeIterator bit = extended_types.begin(method_name, cl);
       bit != extended_types.end(method_name, cl); ++bit)
    {
      if (bit->isSubType(btype))
        {
          hier.insert(bit->getParamType());
        }
    }
}

void
SourceModifierModular::transform_classend(ClassEnd *classend,
                                   ProgElems *modified_source,
                                   ProgElems *modified_meth)
{
  // simply close the class
  modified_source->add_item (new NormalText (classend->getText()));
}

bool
SourceModifierModular::should_use_output_extension(const string &class_name)
{
  return (sourcestructs->contains_multimethods(class_name));
}
