/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <assert.h>

#include "sourcestructs.h"

SourceStructs::~SourceStructs()
{
  ProgElems *progs;
  for (ProgElemsList::const_iterator it = progelems.begin ();
       it != progelems.end (); ++it)
    {
       progs = (*it);
       delete progs;
    }
}

ClassDecl *
SourceStructs::retrieve_class_decl(const string &name) const
{
  SourceStructsIterator sit = find (name);
  if (sit == end ())
    return 0;

  ProgElems *progelems = sit->second;
  return progelems->retrieve_class_decl (name);
}

ProgElems *
SourceStructs::retrieve_source(const string &name) const
{
  SourceStructsIterator sit = find (name);
  if (sit == end ())
    return 0;

  return sit->second;
}

void
SourceStructs::addProgElem(const string &s, ProgElems *p)
{
  insert (valType (s, p));
  addProgElem (p);
}

void
SourceStructs::addProgElem(ProgElems *p)
{
  progelems.insert (p);
}

bool
SourceStructs::contains_multimethods(const string &name) const
{
  ProgElems *source = retrieve_source (name);

  assert (source);

  return source->contains_multimethods();
}
