/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef SOURCESTRUCTS_H
#define SOURCESTRUCTS_H

#include <map>
#include <set>
#include <string>

#include "progelems.hpp"

using std::map;
using std::set;
using std::string;

struct ProgElems_less
{
  typedef std::less<string> value_compare;

  value_compare value_comp;

  bool operator() (const ProgElems *i1, const ProgElems *i2) const
  { return (value_comp (i1->getName (), i2->getName ())); }
};

typedef set<ProgElems *, ProgElems_less> ProgElemsList;

/*
  map: <name of the class, source where it is defined>

  it also contains the sources that do not declare any class
  (e.g., only declarations), but only in progelems.
  This is useful to process files with #includes but with no
  class declaration
*/
class SourceStructs : public map<string, ProgElems *>
{
 protected:
  ProgElemsList progelems;

 public:
  ~SourceStructs();

  // add a source file indexed with a class name defined in it
  void addProgElem(const string &, ProgElems *);

  // simply add a source file
  void addProgElem(ProgElems *);

  ClassDecl *retrieve_class_decl(const string &) const;
  ProgElems *retrieve_source(const string &) const;
  bool contains_multimethods(const string &) const;
  const ProgElemsList &getProgElemsList() const { return progelems; }
};

typedef SourceStructs::const_iterator SourceStructsIterator;
typedef SourceStructs::value_type valType;
typedef ProgElemsList::const_iterator ProgElemsListIterator;

#endif // SOURCESTRUCTS_H
