/***************************************************************************
 *   Copyright (C) 2003 by Lorenzo Bettini                                 *
 *   http://www.lorenzobettini.it                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef WITHDECLTYPE_H
#define WITHDECLTYPE_H

#include "decltype.h"

class WithDeclType
{
 protected:
  DeclType *decl_type;
  bool is_copy;

 public:
  WithDeclType(DeclType *d) : decl_type (d), is_copy (false) {}
  WithDeclType(const WithDeclType &t) : decl_type (t.decl_type), is_copy (true)
  {}
  virtual ~WithDeclType() { /* if (decl_type && ! is_copy) delete decl_type; */ }

  DeclType *getDeclType() const { return decl_type; }
  bool isConst() const { return (decl_type && decl_type->isConst()); }
  bool isPointer() const { return (decl_type && decl_type->isPointer()); }
  const ClassDecl *getClassDecl() const { return (decl_type ? decl_type->getClassDecl() : 0); }
  void setClassDecl(const ClassDecl *c) { if (decl_type) decl_type->setClassDecl(c); }
  const string getTypeName() const { return decl_type->getTypeName(); }
  const string toStringNoConst() const { return decl_type->toStringNoConst(); }
  bool isSubClass(const WithDeclType *t, bool equals_ok = true) const
  {
    return (decl_type && t && decl_type->isSubClass(t->decl_type, equals_ok));
  }
  bool unrelated(const WithDeclType *t) const
  {
    return (decl_type && t && decl_type->unrelated(t->decl_type));
  }
};

#endif // WITHDECLTYPE_H

